@@ -3,151 +3,117 @@ import matplotlib.pyplot as plt
 from matplotlib.patches import Rectangle,Circle,Wedge
 from matplotlib.figure import Figure
 from matplotlib.axes import Axes
+from matplotlib.colors import is_color_like
 
-def show(ar,min=0,max=3,power=1,figsize=(12,12),contrast='std',ax=None,
-         bordercolor=None,borderwidth=5,returnfig=False,cmap='gray',**kwargs):
+def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',min=0,max=4,
+         power=1,bordercolor=None,borderwidth=5,returnfig=False,figax=None,
+         hist=False,n_bins=256,**kwargs):
     """
     General visualization function for 2D arrays.
 
     Accepts:
         ar          (2D array) the array to plot
-        min         (number) minimum display value is set to (median - min*std)
-        max         (number) maximum display value is set to (median + max*std)
-        power       (number) should be <=1; pixel values are set to val**power
         figsize     (2-tuple) size of the plot
-        contrast    (str) determines how image contrast is set. Accepted values
-                    and their behaviors are as follows:
-                        'std'       (default) The min/max values are
-                                    median -/+ n*std, where the two values of n
-                                    are given by the arguments min and max.
+        cmap        (colormap) any matplotlib cmap; default is gray
+        scaling     (str) selects a scaling scheme for the intensity values.
+                    Default is none. Accepted values:
+                        'none'
+                        'log'       values where ar<=0 are set to 0
+                        'power'     requires the 'power' argument be set.
+                                    values where ar<=0 are set to 0
+        clipvals    (str) method for setting clipvalues.  Default is the array
+                    min and max values. Accepted values:
                         'minmax'    The min/max values are np.min(ar)/np.max(r)
-                        'vrange'    The min/max values are set to min/max
-                        'log'       Takes log_10(ar) and visualizes, clipping the
-                                    histogram at the min/max vals. Values <=0 in
-                                    ar are set to 0.
-        ax          (None or 2-tuple) if None, generates a new figure with a single
-                    Axes instance. Otherwise, ax must be a 2-tuple containing
-                    the matplotlib class instances (Figure,Axes), and ar is
-                    plotted in the specified Axes instance.
-                    the passed Axes instance.
-        bordercolor (str or None) if not None, add a border of this color
+
+                        'manual'    The min/max values are set to the values of
+                                    the min,max arguments received by this function
+                        'std'       The min/max values are
+                                            np.median(ar) -/+ N*np.std(ar),
+                                    and N is this functions min,max vals.
+        min         (number) behavior depends on clipvals
+        max         (number)
+        power       (number)
+        bordercolor (str or None) if not None, add a border of this color.
+                    The color can be anything matplotlib recognizes as a color.
         borderwidth (number)
+        returnfig   (bool) if True, the function returns the tuple (figure,axis)
+        figax       (None or 2-tuple) controls which matplotlib Axes object draws
+                    the image.  If None, generates a new figure with a single
+                    Axes instance. Otherwise, ax must be a 2-tuple containing
+                    the matplotlib class instances (Figure,Axes), with ar then
+                    plotted in the specified Axes instance.
+        hist        (bool) if True, instead of plotting a 2D image in ax, plots
+                    a histogram of the intensity values of ar, after any scaling
+                    this function has performed. Plots the clipvals as dashed
+                    vertical lines
+        n_bins      (int) number of hist bins
         **kwargs    any keywords accepted by matplotlib's ax.matshow()
 
     Returns:
         if returnfig==False (default), the figure is plotted and nothing is returned.
         if returnfig==True, return the figure and the axis.
     """
-    if np.min(ar)<0 and power!=1:
-        mask = ar<0
-        _ar = np.abs(ar)**power
-        _ar[mask] = -_ar[mask]
-    elif contrast == 'log':
-        _ar = np.where(ar>0,np.log(ar),0)
-    else:
-        _ar = ar**power
+    assert scaling in ('none','log','power','hist')
+    assert clipvals in ('minmax','manual','std','hist')
 
-    if contrast == 'std':
-        med,std = np.median(_ar),np.std(_ar)
-        vmin = med-min*std
-        vmax = med+max*std
-    elif contrast == 'minmax':
-        vmin = np.min(_ar)
-        vmax = np.max(_ar)
-    elif contrast == 'vrange':
-        vmin = min
-        vmax = max
-    elif contrast == 'log':
-        vmin=np.min(_ar)
-        vmax=np.max(_ar)
+    if scaling == 'none':
+        _ar = ar
+    elif scaling == 'log':
+        mask = ar>0
+        _ar = np.zeros_like(ar)
+        _ar[mask] = np.log(ar[mask])
+    elif scaling == 'power':
+        mask = ar>0
+        _ar = np.zeros_like(ar)
+        _ar[mask] = np.power(ar[mask],power)
     else:
-        raise Exception("Parameter contrast must be 'std' or 'minmax' or 'vrange' or 'log'.")
+        raise Exception
 
-    if ax is None:
+    if clipvals == 'minmax':
+        vmin,vmax = np.min(_ar),np.max(_ar)
+    elif clipvals == 'manual':
+        vmin,vmax = min,max
+    elif clipvals == 'std':
+        m,s = np.median(_ar),np.std(_ar)
+        vmin = m - min*s
+        vmax = m + max*s
+    else:
+        raise Exception
+
+    if figax is None:
         fig,ax = plt.subplots(1,1,figsize=figsize)
     else:
-        fig,ax = ax
+        fig,ax = figax
         assert(isinstance(fig,Figure))
         assert(isinstance(ax,Axes))
 
-    ax.matshow(_ar,vmin=vmin,vmax=vmax,cmap=cmap,**kwargs)
+    if hist:
+        hist,bin_edges = np.histogram(_ar,bins=np.linspace(np.min(_ar),np.max(_ar),
+                                                           num=n_bins))
+        w = bin_edges[1]-bin_edges[0]
+        x = bin_edges[:-1]+w/2.
+        ax.bar(x,hist,width=w)
+        ax.vlines((vmin,vmax),0,ax.get_ylim()[1],color='k',ls='--')
+    else:
+        ax.matshow(_ar,vmin=vmin,vmax=vmax,cmap=cmap,**kwargs)
+
     if bordercolor is not None:
         for s in ['bottom','top','left','right']:
             ax.spines[s].set_color(bordercolor)
             ax.spines[s].set_linewidth(borderwidth)
         ax.set_xticks([])
         ax.set_yticks([])
-    if not returnfig:
-        plt.show()
+
+    if returnfig:
+        return fig,ax
+    elif figax is not None:
         return
     else:
-        return fig,ax
+        plt.show()
+        return
 
-def ax_show(ar,ax,min=0,max=3,power=1,contrast='std',
-            bordercolor=None,borderwidth=5,cmap='gray',returncax=False,**kwargs):
-    """
-    Visualization function which displays the 2D array `ar` in a matplotlib
-    axis instance `ax`. `ax` must be generated and plotted elsewhere.
-
-    Accepts:
-        ar          (2D array) the array to plot
-        ax          (matplotlib.axis) the axis the plot will be displayed in
-        min         (number) minimum display value is set to (median - min*std)
-        max         (number) maximum display value is set to (median + max*std)
-        power       (number) should be <=1; pixel values are set to val**power
-        contrast    (str) determines how image contrast is set. Accepted values
-                    and their behaviors are as follows:
-                        'std'       (default) The min/max values are
-                                    median -/+ n*std, where the two values of n
-                                    are given by the arguments min and max.
-                        'minmax'    The min/max values are np.min(ar)/np.max(r)
-                        'vrange'    The min/max values are set to min/max
-        ax          (None or 2-tuple) if None, generates a new figure with a single
-                    Axes instance. Otherwise, ax must be a 2-tuple containing
-                    the matplotlib class instances (Figure,Axes), and ar is
-                    plotted in the specified Axes instance.
-                    the passed Axes instance.
-        bordercolor (str or None) if not None, add a border of this color
-        borderwidth (number)
-        returncax   (bool) if True, returns the matplotlib.image.AxesImage,
-                    i.e. the image attached to the axis; used to add colorbars
-        **kwargs    any keywords accepted by matplotlib's ax.matshow()
-    """
-    assert isinstance(ax,Axes), "ax must be a matplotlib.axis.Axes instance"
-
-    if np.min(ar)<0 and power!=1:
-        mask = ar<0
-        _ar = np.abs(ar)**power
-        _ar[mask] = -_ar[mask]
-    else:
-        _ar = ar**power
-
-    if contrast == 'std':
-        med,std = np.median(_ar),np.std(_ar)
-        vmin = med-min*std
-        vmax = med+max*std
-    elif contrast == 'minmax':
-        vmin = np.min(_ar)
-        vmax = np.max(_ar)
-    elif contrast == 'vrange':
-        vmin = min
-        vmax = max
-    else:
-        raise Exception("Parameter contrast must be 'std' or 'minmax' or 'vrange'.")
-
-    kwargs.pop('vmin',None),kwargs.pop('vmax',None)
-    ax_image = ax.matshow(_ar,vmin=vmin,vmax=vmax,cmap=cmap,**kwargs)
-    if bordercolor is not None:
-        for s in ['bottom','top','left','right']:
-            ax.spines[s].set_color(bordercolor)
-            ax.spines[s].set_linewidth(borderwidth)
-        ax.set_xticks([])
-        ax.set_yticks([])
-    if returncax:
-        return ax_image
-
-def show_rect(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
-              lims=(0,1,0,1),color='r',fill=True,alpha=1,linewidth=2,**kwargs):
+def show_rect(ar,lims=(0,1,0,1),color='r',fill=True,alpha=0.25,linewidth=2,returnfig=False,
+              **kwargs):
     """
     Visualization function which plots a 2D array with one or more overlayed rectangles.
     lims is specified in the order (x0,xf,y0,yf). The rectangle bounds begin at the upper
@@ -164,7 +130,7 @@ def show_rect(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
 
     Accepts:
         lims        (4-tuple, or list of N 4-tuples) the rectangle bounds (x0,xf,y0,yf)
-        color       (string of list of N strings)
+        color       (valid matplotlib color, or list of N colors)
         fill        (bool or list of N bools) filled in or empty rectangles
         alpha       (number, 0 to 1) transparency
         linewidth   (number)
@@ -183,12 +149,12 @@ def show_rect(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
         assert(all([isinstance(t,tuple) for t in lims]))
         assert(all([len(t)==4 for t in lims]))
         N = len(lims)
-    if isinstance(color,str):
-        color = [color for i in range(N)]
+    if isinstance(color,list):
+        assert len(color)==N
+        assert all([is_color_like(c) for c in color])
     else:
-        assert(isinstance(color,list))
-        assert(len(color)==N)
-        assert(all([isinstance(c,str) for c in color]))
+        assert is_color_like(color)
+        color = [color for i in range(N)]
     if isinstance(fill,bool):
         fill = [fill for i in range(N)]
     else:
@@ -208,7 +174,7 @@ def show_rect(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
         assert(len(linewidth)==N)
         assert(all([isinstance(lw,(float,int,np.float)) for lw in linewidth]))
 
-    fig,ax = show(ar,min,max,power,figsize,returnfig=True,**kwargs)
+    fig,ax = show(ar,returnfig=True,**kwargs)
 
     for i in range(N):
         l,c,f,a,lw = lims[i],color[i],fill[i],alpha[i],linewidth[i]
@@ -222,8 +188,7 @@ def show_rect(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
     else:
         return fig,ax
 
-def show_circ(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
-              center=(0,0),R=10,color='r',fill=True,alpha=1,linewidth=2,**kwargs):
+def show_circ(ar,center,R,color='r',fill=True,alpha=0.3,linewidth=2,returnfig=False,**kwargs):
     """
     Visualization function which plots a 2D array with one or more overlayed circles.
     To overlay one circle, center must be a single 2-tuple.  To overlay N circles,
@@ -236,7 +201,7 @@ def show_circ(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
     Accepts:
         center      (2-tuple, or list of N 2-tuples) the center of the circle (x0,y0)
         R           (number of list of N numbers) the circles radius
-        color       (string of list of N strings)
+        color       (valid matplotlib color, or list of N colors)
         fill        (bool or list of N bools) filled in or empty rectangles
         alpha       (number, 0 to 1) transparency
         linewidth   (number)
@@ -261,12 +226,12 @@ def show_circ(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
         assert(isinstance(R,list))
         assert(len(R)==N)
         assert(all([isinstance(r,(float,int,np.float)) for r in R]))
-    if isinstance(color,str):
-        color = [color for i in range(N)]
+    if isinstance(color,list):
+        assert len(color)==N
+        assert all([is_color_like(c) for c in color])
     else:
-        assert(isinstance(color,list))
-        assert(len(color)==N)
-        assert(all([isinstance(c,str) for c in color]))
+        assert is_color_like(color)
+        color = [color for i in range(N)]
     if isinstance(fill,bool):
         fill = [fill for i in range(N)]
     else:
@@ -287,7 +252,7 @@ def show_circ(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
         assert(all([isinstance(lw,(float,int,np.float)) for lw in linewidth]))
 
 
-    fig,ax = show(ar,min,max,power,figsize,returnfig=True,**kwargs)
+    fig,ax = show(ar,returnfig=True,**kwargs)
 
     for i in range(N):
         cent,r,col,f,a,lw = center[i],R[i],color[i],fill[i],alpha[i],linewidth[i]
@@ -300,8 +265,8 @@ def show_circ(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
     else:
         return fig,ax
 
-def show_annuli(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
-              center=(0,0),Ri=10,Ro=20,color='r',fill=True,alpha=1,linewidth=2,**kwargs):
+def show_annuli(ar,center,Ri,Ro,color='r',fill=True,alpha=0.3,linewidth=2,returnfig=False,
+                **kwargs):
     """
     Visualization function which plots a 2D array with one or more overlayed annuli.
     To overlay one annulus, center must be a single 2-tuple.  To overlay N annuli,
@@ -344,12 +309,12 @@ def show_annuli(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
         assert(isinstance(Ro,list))
         assert(len(Ro)==N)
         assert(all([isinstance(r,(float,int,np.float)) for r in Ro]))
-    if isinstance(color,str):
-        color = [color for i in range(N)]
+    if isinstance(color,list):
+        assert len(color)==N
+        assert all([is_color_like(c) for c in color])
     else:
-        assert(isinstance(color,list))
-        assert(len(color)==N)
-        assert(all([isinstance(c,str) for c in color]))
+        assert is_color_like(color)
+        color = [color for i in range(N)]
     if isinstance(fill,bool):
         fill = [fill for i in range(N)]
     else:
@@ -370,7 +335,7 @@ def show_annuli(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
         assert(all([isinstance(lw,(float,int,np.float)) for lw in linewidth]))
 
 
-    fig,ax = show(ar,min,max,power,figsize,returnfig=True,**kwargs)
+    fig,ax = show(ar,returnfig=True,**kwargs)
 
     for i in range(N):
         cent,ri,ro,col,f,a,lw = center[i],Ri[i],Ro[i],color[i],fill[i],alpha[i],linewidth[i]
@@ -383,8 +348,8 @@ def show_annuli(ar,min=0,max=3,power=1,figsize=(12,12),returnfig=False,
     else:
         return fig,ax
 
-def show_points(ar,x,y,s=1,scale=500,alpha=1,point_color='r',
-                min=0,max=3,power=1,figsize=(12,12),returnfig=False,**kwargs):
+def show_points(ar,x,y,s=1,scale=50,alpha=1,point_color='r',
+                figsize=(12,12),returnfig=False,**kwargs):
     """
     Visualization function which plots a 2D array with one or more points.
     x and y are the point centers and must have the same length, N.
@@ -404,7 +369,7 @@ def show_points(ar,x,y,s=1,scale=500,alpha=1,point_color='r',
         If returnfig==False, the figure and its one axis are returned, and can be
         further edited.
     """
-    fig,ax = show(ar,min,max,power,figsize,returnfig=True,**kwargs)
+    fig,ax = show(ar,figsize,returnfig=True,**kwargs)
     ax.scatter(y,x,s=s*scale/np.max(s),color=point_color)
 
     if not returnfig:
